# Scripting

## What is a script?

* A Bash script is a text file containing a sequence of commands.  
* It automates tasks that you would normally run manually in the terminal.  
* It is interpreted, not compiled, meaning Bash reads and executes it line by line.  
* Used for system administration, automation, and data processing in Unix/Linux environments.


## Script skeleton

The first step is to **create a file** e.g. `script.sh` using the editor of your choice (vscode,sublime,gedit,etc) or by command line (vi,vim,nano,emacs,etc).  

A good practice is to use suffix `.sh` to tell yourself that this file contain ba**sh** code.


### shebang

The good practice is to use a shebang.  
A **shebang** (#!) is the first line of a file specifying the interpreter to use to translate the code into machine instructions.  

If you read a german text and interpret is as english it does not make any sense... it is the same in programmation!  
Here we will program in **bash**, so we must tell the machine that the code contained in our file has to be read in bash!  

How to do so? We add as the first line of your file a shebang with the path to the interpreter:
```bash
#!/bin/bash
```

!!! note
    With a shebang, you can execute the script directly (if it has execution permissions) using `./script.sh`.  
    Without a shebang, you must specify the interpreter explicitly when running the script: `bash script.sh`.


### Executable

A good practice is to provide the execute permissions to that file running:

```bash
chmod +x script.sh
```

!!! note
    check permission before and after using `ls -l script.sh` to observe the difference.

Here you go, you can now execute your file calling its name `./script.sh` (if in the current directory).  

### Code

Your script can contain commands, variables, loops, conditions, and functions to create complex workflows.  

Add a command in your file
```bash
#!/bin/bash

echo hello
```

and execute it `./script.sh`

```bash
$ coucou
```

Here it is you have written and executer your first script!

### Comment

Anywhere in your code you can add comment.
Comment start by `#`. The only excpetion to this rule is the shebang.

Comments are essential for making your script readable and understandable by others (or even your future self). They provide explanations about the code’s purpose and functionality, making maintenance and debugging easier.

```bash
#!/bin/bash

# This is a comment
echo hello
```

### Exit

The `exit`command allows the program to quit prematurely.

```bash
#!/bin/bash

# stop here
exit

# This is a comment
echo hello # <= this will never be executed
```

In Bash, exit codes (also known as return codes or status codes) indicate the success or failure of a command or script.  
The best practice is to define an exit code to inform users or other scripts of how things have gone.  
Here are some common exit codes:  

| exit code | explanation |
|----------|----------|
| 0 | Success (no error).
| 1 | General error (catch-all for miscellaneous errors).
| 2 | Incorrect usage (misuse of built-in shell commands).
| 126 | Command invoked cannot be executed (e.g., permission issue).
| 127 | Command not found.
| 128 | Invalid argument to exit.
| 130 | Script terminated by Ctrl+C (SIGINT).
| 137 | Process killed (usually by kill -9, SIGKILL).
| 139 | Segmentation fault.
| 143 | Process terminated by SIGTERM (default termination signal).
| 255 | Exit status out of range (must be between 0 and 255).


!!! Note 
    Each command in Bash returns an exit code, which can be accessed using `$?`

!!! Success "Quick recap"
    In this section we've learned:

    - What is a script
    - The good practices to write and execute a script:  
            - the use of shebang  
            - make script executable  
            - executing it using `./script.sh`  
